import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';
import 'package:modular_system_core/src/widget/entity_table/table_tile.dart';
export './viewmodel.dart';
export './app_bar.dart';

class EntityTable<T> extends StatefulWidget {
  const EntityTable({
    Key? key,
    required this.context,
    required this.appViewModel,
    required this.actions,
    required this.onUpdate,
    required this.itemBuilder,
    required this.formBuilder,
    required this.filtersSheetBuilder,
  }) : super(key: key);

  // injected
  final BuildContext context;
  final AppViewModel appViewModel;
  final List<DataTableAction> actions;
  // callbacks
  final Future<KeyValuePair<PaginationOut, List<T>>> Function(WebSearchRequest search) onUpdate;
  // builders
  final Widget Function(EntityTableViewModel viewModel, dynamic item) itemBuilder;
  final Widget Function(EntityTableViewModel viewModel) formBuilder;
  final Widget Function(EntityTableViewModel viewModel) filtersSheetBuilder;

  @override
  State<EntityTable> createState() => _EntityTableState<T>();
}

class _EntityTableState<T> extends State<EntityTable> {
  late EntityTableViewModel<T> viewModel;

  bool isLoading = false;
  late Widget body = viewModel.state.body;

  @override
  void initState() {
    super.initState();
    viewModel = EntityTableViewModel<T>(
      context: context,
      appViewModel: widget.appViewModel,
      actions: widget.actions,
      onUpdate: onUpdate,
      itemBuilder: itemBuilder,
      formBuilder: widget.formBuilder,
      filtersSheetBuilder: widget.filtersSheetBuilder,
    );
    viewModel.state.subscribe(onStateChanged);
  }

  @override
  void dispose() {
    viewModel.state.unsubscribe(onStateChanged);
    super.dispose();
  }

  Future<KeyValuePair<PaginationOut, List<T>>> onUpdate(WebSearchRequest searchRequest) async {
    var data = await widget.onUpdate(searchRequest);
    return data as KeyValuePair<PaginationOut, List<T>>;
  }

  Widget itemBuilder(EntityTableViewModel<T> vm, T data) {
    return widget.itemBuilder(vm, data);
  }

  void onStateChanged(TableState state) {
    setState(() {
      isLoading = state.isLoading;
      body = state.body;
    });
  }

  @override
  Widget build(BuildContext context) {
    return body;
  }
}

class EntityTableBody<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableBody(this.viewModel);

  final EntityTableViewModel<T> viewModel;

  @override
  State<EntityTableBody> createState() {
    return _EntityTableBodyState<T>();
  }
}

class _EntityTableBodyState<T> extends State<EntityTableBody> {
  late bool isLoading = widget.viewModel.state.isLoading;
  late List data = widget.viewModel.state.data;

  @override
  void initState() {
    super.initState();
    subscribeToEvents();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  void subscribeToEvents() {
    widget.viewModel.state.subscribe(onStateChanged);
  }

  void unsubscribeToEvents() {
    widget.viewModel.state.unsubscribe(onStateChanged);
  }

  void onStateChanged(TableState state) {
    setState(() {
      isLoading = state.isLoading;
      data = state.data;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: widget.viewModel.appViewModel.colors.background,
      child: CustomScrollView(
        shrinkWrap: isLoading,
        slivers: [
          EntityTableAppBar(widget.viewModel),
          body(),
        ],
      ),
    );
  }

  Widget body() {
    if (isLoading) {
      return loading();
    }
    return SliverList(
      key: UniqueKey(),
      delegate: SliverChildBuilderDelegate(
        (context, index) {
          var item = data[index];
          return TableTile<T>(
            index: index,
            viewModel: widget.viewModel as EntityTableViewModel<T>,
            child: widget.viewModel.itemBuilder(widget.viewModel, item),
          );
        },
        childCount: data.length,
      ),
    );
  }

  Widget loading() {
    return SliverToBoxAdapter(
      child: Center(
        child: widget.viewModel.appViewModel.loader.getLoader(),
      ),
    );
  }
}
